use strict;
use ContractUtil;
use Storable;

####################################################################
# Mark information from GDB - currently focussed on maintenance info
package GdbMarkDetail;
use base 'AutoHash';
use DBI qw/:sql_types/;

package GdbMarkDetailList;

# Query to get stuff out of GDB
our $mark_detail_sql = <<END_SQL;
with code as (
    select
        mrk.nod_id,
        mrk.id as mrk_id,
        mkn.name as code
    from 
        bde.crs_mark mrk
        join bde.crs_mark_name mkn on mkn.mrk_id=mrk.id and mkn.type='CODE'
        join bde.crs_node nod on nod.id=mrk.nod_id and nod.status='AUTH'
    where
        mkn.name=?
        and mrk.status='COMM'
),
img as (
    select
        distinct on (c.nod_id)
        c.nod_id,
        image_id as site_photo_id
    from
        code c
        join gdb.gdb_image gi on gi.geodetic_code=c.code and gi.type='SITE'
    order by
        c.nod_id,
        gi.date desc
) ,
crd49 as 
(
    select 
        c.nod_id,
        cor.display as order49
    from
        code c
        join crs_coordinate coo on coo.nod_id=c.nod_id and coo.status='AUTH'
        join crs_coordinate_sys cos on cos.id=coo.cos_id and cos.dtm_id=18 -- NZGD1949 crs_datum
        join crs_cord_order cor on cor.id=coo.cor_id
),
crdh as 
(
    select 
        c.nod_id,
        min(cor.display) as orderh
    from
        code c
        join crs_coordinate coo on coo.nod_id=c.nod_id and coo.status='AUTH'
        join crs_coordinate_sys cos on cos.id=coo.cos_id
        join crs_datum dtm on dtm.id=cos.dtm_id and dtm.type='GRAV'
        join crs_cord_order cor on cor.id=coo.cor_id
    group by 
        c.nod_id
)
select
    DISTINCT ON (c.nod_id)
    c.nod_id,
    c.mrk_id,
    c.code AS "CODE",
    mkn2.name as "NAME",
    mkn3.name as "ALTN",
    coo.value1 AS "CRDLAT",
    coo.value2 as "CRDLON",
    coo.value3 as "CRDHGT",
    cor.display as "CRDORD",
    crd49.order49 as "CRDORD49",
    crdh.orderh as "HGTORD",
    mrk.type as "MRKT",
    mpsm.condition as "MPSC",
    mrk.beacon_type as "MRKE",
    mrk.protection_type as "MRKR",
    mrk.desc as "MRKD",
    loc.name as "LandDistrict",
    CASE WHEN gmd.geodetic_code IS NULL THEN 'N' ELSE 'Y' END as have_mark_details,
    gmd.id as gmd_id,
    to_char(gmd.maint_date,'YYYY.MM.DD') as "MDAT",
    mrk.status as "MRKS",
    gmd.plan_refs as "PLRF",
    to_char(gmd.mark_date,'YYYY.MM.DD') as "EDAT",
    gmd.prot2_type as "MRKR2",
    gmd.glrel as "GLREL",
    gmd.bcon_hgt1 AS "BCNHGT",
    gmd.bcon_hgt2 AS "BCNHGT2",
    gmd.bcon_hgt3 AS "BCNHGT3",
    gmd.bcon_hgt4 AS "BCNHGT4",
    gmd.bcon_hgt5 AS "BCNHGT5",
    gmd.bcon_hgt6 AS "BCNHGT6",
    to_char(gmd.bcon_date,'YYYY.MM.DD') as "BDAT",
    gmd.bcon_ecc AS "BECC",
    gmd.access_note AS "MLOC",
    gmd.plaque_exists AS "PLQEXIST",
    gmd.plate_exists as "PLTEXIST",
    gmd.info_exists as "INFOEXIST",
    gmd.mark_mntdone as "MPSM",
    gmd.bcon_mntdone as "MPSB",
    gmd.prot_mntdone as "MPSP",
    gmd.mark_mntreq as "MDMK",
    gmd.bcon_mntreq AS "MDBE",
    gmd.prot_mntreq AS "MDPR",
    gmd.occupier AS "OWNR",
    gmd.occ_phone as "PHNO",
    gmd.occ_address as "PADD",
    gmd.access_restrict AS "ARES",
    gmd.gps_use_code AS "GNSSU",
    gmd.cell_coverage AS "CELL",
    gmd.comments as "COMM",
    img.site_photo_id
FROM
    code c
    left outer join gdb.gdb_mark_detail gmd on gmd.geodetic_code=c.code
    join bde.crs_mark mrk ON mrk.id=c.mrk_id

    left outer join bde.crs_mark_name mkn2 on mkn2.mrk_id = mrk.id and mkn2.type = 'CURR' 
    left outer join bde.crs_mark_name mkn3 on mkn3.mrk_id = mrk.id and mkn3.type = 'ALTE' 

    join bde.crs_node nod ON nod.id = mrk.nod_id 
    left outer join crs_site_locality slo on slo.sit_id = nod.sit_id 
    left outer join crs_locality loc on loc.id = slo.loc_id and loc.type = 'LDST' 
    join crs_coordinate_sys cos ON cos.id = nod.cos_id_official
    join crs_coordinate coo ON coo.cos_id = nod.cos_id_official and coo.nod_id = nod.id and coo.status='AUTH'
    join crs_cord_order cor ON cor.id = coo.cor_id
    left outer join crs_mrk_phys_state mpsm on  
        mpsm.mrk_id = mrk.id and mpsm.type = 'MARK' and mpsm.status='CURR' 
    left outer join crs_mrk_phys_state mpsb on  
        mpsb.mrk_id = mrk.id and mpsb.type = 'BCON' and mpsb.status='CURR' 
    left outer join crs_mrk_phys_state mpsp on  
        mpsp.mrk_id = mrk.id and mpsp.type = 'PROT' and mpsp.status='CURR'  
    left outer join img on img.nod_id=c.nod_id
    left outer join crd49 on crd49.nod_id=c.nod_id
    left outer join crdh on crdh.nod_id=c.nod_id
ORDER BY
  c.nod_id,
  gmd.maint_date desc
END_SQL

sub new
{
    # Options can include
    #   connect=>0,1  - attempt to connect to the geodetic database
    #   clear_cache=0,1 - attempt to clear the cache
    my($class,$cfg,%options) = @_;
    my $dbh;
    my $sth;
    my $connect=exists $options{connect} ? $options{connect} : 1;
    my $clearcache=$options{clear_cache};
    if( ! defined($connect) || $connect )
    {
        $connect=1;
        eval
        {
            require GDBConnection;
            $dbh = GDBConnection::Connect();
            $sth = $dbh->prepare($mark_detail_sql);
        };
        if( $@ )
        {
            print("Using online geodetic database\n");
            $connect=0;
        }
    }
    my $expiry=$cfg->get('gdbmarkdetail_cache_expiry',0.5)*60*60*24;
    my $self= bless {
        dbh => $dbh,
        sth => $sth,
        marks => {},
        cfg => $cfg,
        expiry => $expiry
    }, $class;

    $self->clearcache() if $clearcache;
    $self->loadcache();
    return $self;
}

sub DESTROY
{
    my($self) = @_;
    $self->savecache();
}

sub connected { return $_[0]->{sth} ? 1 : 0; }

sub create_mark
{
    my($self,$markdata, $code) = @_;
    my $mark;
    if( $markdata )
    {
        $code = $markdata->{CODE} if ! $code;
        $mark = new GdbMarkDetail( 
            %$markdata, 
            EXMK=>'Y',
            # Additional fields mapping to MDFC fields
            ORDV1=>$markdata->{CRDLAT},
            ORDV2=>$markdata->{CRDLON},
            ORDV3=>$markdata->{CRDHGT},
            CROD=>$markdata->{CRDORD},
            # Fixing up land district
            DISTRICT=>$self->{cfg}->unmap('DISTRICT',$markdata->{LandDistrict}),
            # Source database
            _dbh=>$self->{dbh},
            );
           
    }
    elsif( $code )
    {
        $mark = new GdbMarkDetail( CODE=>$code, EXMK=>'N' );
    }

    $mark->{_expiry}=time()+$self->{expiry};
    $self->{marks}->{$code} = $mark if $mark;
    return $mark;
}

sub cachefile
{
    my($self)=@_;
    return '' if $ENV{GDBMARK_NO_CACHE};
    my $dir=$ENV{TEMP} || $ENV{TMP} || $ENV{HOME} || ".";
    my $cachename=$self->{cfg}->get('gdbmarkdetail_cache',$dir.'/.gdbmrkdtl.cache');
    return $cachename;
}

sub clearcache
{
    my($self)=@_;
    my $cachefile=$self->cachefile();
    unlink($cachefile) if $cachefile;
}

sub loadcache
{
    my($self)=@_;
    return if %{$self->{marks}};
    my $cachefile=$self->cachefile();
    return if ! $cachefile;
    eval
    {
        my $cache = Storable::retrieve($cachefile);
        my $now=time();
        foreach my $code (keys %$cache)
        {
            delete $cache->{$code} if $cache->{$code}->{_expiry} < $now;
        }
        $self->{marks} = $cache;
    };
}

sub savecache
{
    my($self)=@_;
    my $cachefile=$self->cachefile();
    return if ! %{$self->{marks}} || ! $cachefile;
    eval
    {
        my $cache=$self->{marks};
        Storable::store($cache,$cachefile);
    }
}

sub loadcodes_url
{
    my($self,$codes) = @_;
    print "Fetching ",join(" ",@$codes),"\n";
    if( ! $self->{sth} )
    {
        eval
        {
            require LWP::Simple;
            require IO::String;
            require CSV_File;
            my $url=$self->{cfg}->get('gdbmarkdetail_url');
            my $codelist=join('+',@$codes);
            $url=~ s/{codelist}/$codelist/;
            my $csvdata = LWP::Simple::get($url);
            my $csvfh=IO::String->new($csvdata);
            my $csv=CSV_File->new($csvfh);
            while( $csv->NextRecord )
            {
                my $markdata=$csv->recordhash;
                $self->create_mark($markdata);
            }
        }
    }
}

sub loadcodes_sth
{
    my($self,$codes) = @_;
    my $sth = $self->{sth};
    my $markdata;
    if( $sth )
    {
        foreach my $code (@$codes)
        {
            $sth->execute($code);
            $markdata = $sth->fetchrow_hashref();
            $self->create_mark( $markdata, $code );
        }
    }
}

sub loadcodes
{
    my($self,$codes) = @_;
    my $codelist=[];
    foreach my $code (@$codes)
    {
        push(@$codelist,$code) if ! exists $self->{marks}->{$code};
    }
    if( @$codelist )
    {
        if( $self->{sth} ) { $self->loadcodes_sth($codelist); }
        else { $self->loadcodes_url($codelist); }
    }
    foreach my $code (@$codes)
    {
        $self->create_mark(undef,$code) if ! exists $self->{marks}->{$code};
    }
}

sub markinfo
{
    my($self,$code) = @_;
    $code = uc($code);
    $self->loadcodes([$code]) if ! exists $self->{marks}->{$code};
    return $self->{marks}->{$code};
}

1;
